#pragma once

#include "ircmsg.hpp"
#include "irc_command.hpp"

#include <boost/asio.hpp>
#include <boost/signals2.hpp>

#include <list>
#include <memory>
#include <string>

class Connection : public std::enable_shared_from_this<Connection>
{
private:
    boost::asio::ip::tcp::socket stream_;
    boost::asio::steady_timer write_timer_;
    std::list<std::string> write_strings_;

    auto writer() -> void;
    auto writer_immediate() -> void;
    auto dispatch_line(char * line) -> void;

public:
    Connection(boost::asio::io_context & io);

    boost::signals2::signal<void()> sig_connect;
    boost::signals2::signal<void()> sig_disconnect;
    boost::signals2::signal<void(IrcCommand, const IrcMsg &)> sig_ircmsg;

    auto get_executor() -> boost::asio::any_io_executor {
        return stream_.get_executor();
    }

    /// Write bytes into the socket. Messages should be properly newline terminated.
    auto write_line(std::string message) -> void;

    auto connect(
        boost::asio::io_context & io,
        std::string host,
        std::string port
    ) -> boost::asio::awaitable<void>;

    auto close() -> void;
};

