#pragma once

#include <boost/signals2.hpp>

#include <memory>
#include <regex>
#include <string_view>
#include <variant>

class Connection;
struct hs_database;
struct hs_scratch;

enum class SnoteTag
{
    ClientConnecting,
    ClientExiting,
    RejectingKlined,
    NickChange,
    CreateChannel,
    TemporaryKlineExpired,
    PropagatedBanExpired,
    DisconnectingKlined,
    NewPropagatedKline,
    NewTemporaryKline,
    LoginAttempts,
    PossibleFlooder,
    Killed,
    TooManyGlobalConnections,
    SetVhostOnMarkedAccount,
};

class SnoteMatch
{
public:
    SnoteMatch(std::regex const& regex, std::string_view full)
    : components_{std::make_pair(std::ref(regex), full)}
    {}

    auto get_results() -> std::match_results<std::string_view::const_iterator> const&;

private:
    SnoteTag tag_;
    std::variant<std::pair<std::regex const&, std::string_view>, std::match_results<std::string_view::const_iterator>> components_;
};

struct SnoteThread
{
    struct DbDeleter
    {
        auto operator()(hs_database * db) const -> void;
    };

    struct ScratchDeleter
    {
        auto operator()(hs_scratch * scratch) const -> void;
    };

    /// @brief Database of server notice patterns
    std::unique_ptr<hs_database, DbDeleter> db_;

    /// @brief HyperScan scratch space
    std::unique_ptr<hs_scratch, ScratchDeleter> scratch_;

    boost::signals2::signal<void(SnoteTag, SnoteMatch&)> sig_snote;

    static auto start(Connection& connection) -> std::shared_ptr<SnoteThread>;
};
